#include "config.h"

#define DQ P00

u16 Temp_Val;//温度值

//单总线延时函数
void delay_us(uint32_t us) {
    unsigned long edata i;

    while(--us) {
        _nop_();
        _nop_();
        _nop_();
        i = 4UL;
        while (i) i--;        
    }
}

//单总线变量初始化函数
void DS18B20_Tag_Init(void) {
    Temp_Val=0;//初始化温度值
}

//单总线初始化函数
//0：初始化成功 1：初始化失败
bit DS18B20_Init(void) {
    static bit ack=0;//应答信号
    DQ = 0;//拉低总线
    delay_us(500);//延时480us以上
    DQ = 1;//释放总线
    delay_us(60);//延时大于60us,确保能收到DS18B20发送的60-240us的存在脉冲
    ack = DQ;//读取单总线上的状态
    delay_us(240);//延时240us,让DS18B20释放总线
    DQ=1;//释放总线
    return ack;//返回应答信号
}

//单总线写一个字节
void DS18B20_WriteByte(uint8_t dat) {
    u8 i;//计数变量

    for(i=0;i<8;i++) {
        DQ = 0;//拉低总线,产生写时隙
        delay_us(2);//每次读写都要间隔大于1us
        DQ=1;//15us内释放总线(写1时序)
        delay_us(2);//稍作延时
        DQ=dat&0x01;//往单总线上写数据(LSB)
        delay_us(45);//在15-60us内等待DS18B20采集数据
        DQ=1;//释放总线
        dat>>=1;//右移一位,准备下一次发送
    }
}

//单总线读一个字节
uint8_t DS18B20_ReadByte(void) {
    u8 i;//计数变量
    u8 dat;//读取的数据

    for(i=0;i<8;i++) {
        DQ = 0;//拉低总线,产生读时隙
        delay_us(2);//稍作延时
        DQ = 1;//释放总线
        dat>>=1;//右移一位,准备接收数据
        if(DQ) dat|=0x80;//读取数据(LSB),要读取最高位
        delay_us(60);//延时60us
        DQ=1;//释放总线
        delay_us(2);//稍作延时
    }
    return dat;
}

//读取温度
float DS18B20_ReadTemp(void) {
    float temp;//温度暂存变量(浮点型)
    u8 temp1,temp2;//温度低8位 温度高8位

    DS18B20_Init();//总线初始化函数
    DS18B20_WriteByte(0xcc);//忽略ROM
    DS18B20_WriteByte(0x44);//开始温度转换
    while(!DQ);//一定要等待转换完成,否则读取失败

    DS18B20_Init();//总线初始化
    DS18B20_WriteByte(0xcc);//忽略ROM
    DS18B20_WriteByte(0xbe);//读取刮擦板
    temp1=DS18B20_ReadByte();//读取温度低8位
    temp2=DS18B20_ReadByte();//读取温度高8位
    temp=(float)((temp1+temp2*256)*0.0625);//将温度数据合并为16位,并缩小16倍,获得温度值
    return temp;//返回温度值(浮点型)
}

//温度值CDC串口返回函数
void CDC_Temp_Check() {
    static u8 xdata Temp_Text[100];//格式化数组
    static u8 Init_Flag = 0;//单总线初始化状态监视位
    const u8 Init_Sucess[]="DS18B20初始化成功";
    const u8 Init_Error[]="DS18B20初始化失败";

    Init_Flag=DS18B20_Init(); //读取初始化状态
    switch(Init_Flag) {
        case 0: 
            sprintf(Temp_Text,"%s\r\n",Init_Sucess);
            printf(Temp_Text);
            break;
        case 1: 
            sprintf(Temp_Text,"%s\r\n",Init_Error);
            printf(Temp_Text);
            break;
        default: 
            break;
    }
    Temp_Val=DS18B20_ReadTemp();//获取温度值
    printf("Temp_Val=%u\r\n",Temp_Val);//打印温度值
}